function [xmin,fmin,fcount] = DSSA(n,U,L,f,maxi)
% *******************************************************
% DSSA = Direct Search Simulated Annealing 
% The DSSA Code is created on Oct. 16, 2001 by A. Hedar 
% Modified on Aug. 13, 2005
% Reference: 
%       A. Hedar and M. Fukushima,Hybrid simulated annealing and direct   
%       search method for nonlinear unconstrained global optimization, 
%       Optimization Methods and Software, 17 (2002) 891-912.
% *******************************************************
% Inputs:
%  n    = Number of the objective variables   
%  U    = n-Dimension Vector of upper limits of the objective variables
%  L    = n-Dimension Vector of lower limits of the objective variables
%  f    = Objective function
%  dsn1 = PS pruned direction set in case of getting a descent direction
%  dsn2 = PS pruned direction set in case of not getting a descent direction
%
% Outputs:
%  xmin   = Best point obtained by SAHPS 
%  fmin   = Best function value obtained by SAHPS
%  fcount = Number of function evaluations                   
%   
%
%Initialize counters
%
fcount=0;				%number of functional evaluations
itc=0;	        		%number of iterations
%
%Set the parametars	
%
%
% dsize=size(x0); n=dsize(1);
%
% Generation of the initial point
rang = max(U-L);
center = (U+L)/2;
x=zeros(n,1);
for i=1:n;
    x0(i)=L(i)+rand*(U(i)-L(i));
end
x1=2*center-x0;
%
%Set the parametars	
%
if nargin < 5 maxi=50*n; end
rd = 0.5;         %reduction factor of temp
maxrit = n;       %number of iterations before reducte the temp
tol = 1.0D-8;     %termanation acurcy
nbest = n;        %number of best stored functional values
rad = 0.001;      %the redius of the balls in the best points List
mloopc = 2;		  %the number of iterating the main loop
edger = 0.5;	  %reduction factor of edge length for simplex
%
%creating the initial simplex
%
x=zeros(n,n+1);
x(:,1)=x0;
id=eye(n);
edgel=min(0.1*max(U-L),4);
edgebest=edgel;
df=0;
fv(1)=feval(f,x(:,1));
fcount=fcount+1;
edge=edgel*ones(n,1);
for j=2:n+1; x(:,j)=x0'+edge(j-1)*id(:,j-1); end
for j=2:n+1; fv(j)=feval(f,x(:,j)); end;
fcount=fcount+n;
for j=1:n; d(j)=abs(fv(1)-fv(j+1)); end
df=norm(d);
while(df<10 & edgel<4)
   edgel=2*edgel;
   edge=edgel*ones(n,1);
   for j=2:n+1; x(:,j)=x0'+edge(j-1)*id(:,j-1); end
   for j=2:n+1; fv(j)=feval(f,x(:,j)); end;
   fcount=fcount+n;
   for j=1:n; d(j)=abs(fv(1)-fv(j+1)); end
   df=norm(d);          
end
%
%Order the vertices for the first time
%
xtmp=zeros(n,n+1); 
[fs,is]=sort(fv); xtmp=x(:,is); x=xtmp; fv=fs;
maxdf=fv(n+1)-fv(1);
itc=itc+1;
% Tmax=(fv(n+1)-fv(1))/log(1/0.9);
% Tmin=0.00001*Tmax;
Tmax=max(0.001,(fv(n+1)-fv(1)))/log(1/0.9);
Tmin=0.00001*Tmax;
T=Tmax;
for j=1:nbest; bestxl(:,j)=x(:,1); bestfl(j)=fv(1);end
%
% Main loop
%
for loopc=1:mloopc;
% while(edgel>0.0000001)
   while(itc < maxi & T > Tmin)  
      rditc=0;
      while(rditc < maxrit)
         r=1;            %number of the reflecting vertices
         accept=0;
         while(accept == 0)
            p=n-r+1;
            one2p=1:p;
            xbar=sum(x(:,one2p),2)/p;     %centriod of p best point
            for j=1:r
               k=j+p;
               xr(:,j)=xbar+(0.2*rand+0.9)*(xbar-x(:,k));
            end
            for j=1:r; fr(j)=feval(f,xr(:,j)); end
            fcount=fcount+r;
            [frs,irs]=sort(fr); ii=irs(1);
            frmin=frs(1); xrmin=xr(:,ii);
            prob=exp(-(frmin-fv(1))/T);
            if frmin < fv(1)
               for j=1:r; k=j+p; x(:,k)=xr(j); end
               bestla=0;
               jj=1;
               while(bestla~=1 & jj < nbest)
                  v(:,jj)=xrmin-bestxl(:,jj);
                  if norm(v(:,jj)) < rad
                     bestla=1;
                  end
                  jj=jj+1;
               end
               if frmin < bestfl(nbest) & bestla~=1 
                  bestxl(:,nbest)=xrmin;
                  bestfl(nbest)=frmin;
                  [bestls,bestis]=sort(bestfl);
                  xltmp=bestxl(:,bestis); bestxl=xltmp;
                  bestfl=bestls;
               end
               for j=1:r; k=j+p; x(:,k)=xr(:,j); fv(k)=fr(j); end
               accept=1;
            elseif prob >= rand;
               for j=1:r; k=j+p; x(:,k)=xr(j); fv(k)=fr(j); end
               accept=2;
            elseif r==n
               for j=1:r; k=j+p; x(:,k)=xr(j); fv(k)=fr(j); end
               accept=3;
            else
               r=r+1;
               accept=0;
            end
         end
         xtmp=zeros(n,n+1);
         [fs,is]=sort(fv); xtmp=x(:,is); x=xtmp; fv=fs;
         maxdf=fv(n+1)-fv(1);
         rditc=rditc+1;
         itc=itc+1;         
      end
      T=T*rd;      
   end
   xnew(:,1)=x1;
   for j=2:n+1; xnew(:,j)=xnew(:,1)+edge(j-1)*id(:,j-1); end
   x=xnew;
   for j=1:n+1; fv(j)=feval(f,x(:,j)); end;
   fcount=fcount+n+1;
   xtmp=zeros(n,n+1);
   [fs,is]=sort(fv); xtmp=x(:,is); x=xtmp; fv=fs;
   T=Tmax;
   maxdf=fv(n+1)-fv(1);
   itc=0;
end
finalbestxl(:,1)=bestxl(:,1);
finalbestfl(1)=bestfl(1);
finalnbest=1;
for j=2:nbest;
   accept=0;
   for i=1:j-1;
      if abs(bestxl(:,j)-bestxl(:,i))<0.1
         accept=1;
      end
   end
   if accept==0;
      finalnbest=finalnbest+1;
      finalbestxl(:,finalnbest)=bestxl(:,j);
      finalbestfl(finalnbest)=bestfl(j);
   end   
end
xmin=finalbestxl(:,1);
fmin=bestfl(1);
edge=edgebest*ones(n);
for k=1:nbest;
    x1=zeros(n,n+1);
    x1(:,1)=bestxl(:,k);
    id=eye(n);
    for j=2:n+1; x1(:,j)=x1(:,1)+edge(j-1)*id(:,j-1); end
    [x,fcoun,f1]=nelder(x1,f,tol);
    if f1 < fmin
        xmin=x(:,1);
        fmin=f1;
    end
    fcount=fcount+fcoun;
end             
